<?php
/**
 * Created by PhpStorm.
 * User: Siteguarding.com
 * Date: 10/19/2017
 * Time: 5:14 AM
 */

class HTTPClient
{
    var $agent;
    var $http;
    var $timeout;
    var $cookies;
    var $referer;
    var $max_redirect;
    var $max_bodysize;
    var $header_regexp;
    var $headers;
    var $debug;
    var $start = 0;
    var $error;
    var $redirect_count;
    var $resp_status;
    var $resp_body;
    var $resp_headers;
    var $user;
    var $pass;
    var $proxy_host;
    var $proxy_port;
    var $proxy_user;
    var $proxy_pass;
    var $proxy_ssl;

    function HTTPClient()
    {
        $this->agent = 'Mozilla/4.0 (compatible; DokuWiki HTTP Client; ' . PHP_OS . ')';
        $this->timeout = 15;
        $this->cookies = array();
        $this->referer = '';
        $this->max_redirect = 3;
        $this->redirect_count = 0;
        $this->status = 0;
        $this->headers = array();
        $this->http = '1.0';
        $this->debug = false;
        $this->max_bodysize = 0;
        $this->header_regexp = '';
        if (extension_loaded('zlib')) $this->headers['Accept-encoding'] = 'gzip';
        $this->headers['Accept'] = 'text/xml,application/xml,application/xhtml+xml,' . 'text/html,text/plain,image/png,image/jpeg,image/gif,*/*';
        $this->headers['Accept-Language'] = 'en-us';
    }

    function get($url, $sloppy304 = false)
    {
        if (!$this->sendRequest($url)) return false;
        if ($this->status == 304 && $sloppy304) return $this->resp_body;
        if ($this->status != 200) return false;
        return $this->resp_body;
    }

    function post($url, $data)
    {
        if (!$this->sendRequest($url, $data, 'POST')) return false;
        if ($this->status != 200) return false;
        return $this->resp_body;
    }

    function sendRequest($url, $data = '', $method = 'GET')
    {
        $this->start = $this->_time();
        $this->error = '';
        $this->status = 0;
        $uri = parse_url($url);
        $server = $uri['host'];
        $path = $uri['path'];
        if (empty($path)) $path = '/';
        if (!empty($uri['query'])) $path .= '?' . $uri['query'];
        $port = $uri['port'];
        if ($uri['user']) $this->user = $uri['user'];
        if ($uri['pass']) $this->pass = $uri['pass'];
        if ($this->proxy_host) {
            $request_url = $url;
            $server = $this->proxy_host;
            $port = $this->proxy_port;
            if (empty($port)) $port = 8080;
        } else {
            $request_url = $path;
            $server = $server;
            if (empty($port)) $port = ($uri['scheme'] == 'https') ? 443 : 80;
        }
        if ($port == 443 || $this->proxy_ssl) $server = 'ssl://' . $server;
        $headers = $this->headers;
        $headers['Host'] = $uri['host'];
        $headers['User-Agent'] = $this->agent;
        $headers['Referer'] = $this->referer;
        $headers['Connection'] = 'Close';
        if ($method == 'POST') {
            if (is_array($data)) {
                $headers['Content-Type'] = 'application/x-www-form-urlencoded';
                $data = $this->_postEncode($data);
            }
            $headers['Content-Length'] = strlen($data);
            $rmethod = 'POST';
        } elseif ($method == 'GET') {
            $data = '';
        }
        if ($this->user) {
            $headers['Authorization'] = 'Basic ' . base64_encode($this->user . ':' . $this->pass);
        }
        if ($this->proxy_user) {
            $headers['Proxy-Authorization'] = 'Basic ' . base64_encode($this->proxy_user . ':' . $this->proxy_pass);
        }
        $start = time();
        $socket = @fsockopen($server, $port, $errno, $errstr, $this->timeout);
        if (!$socket) {
            $resp->status = '-100';
            $this->error = "Could not connect to $server:$port\n$errstr ($errno)";
            return false;
        }
        stream_set_blocking($socket, 0);
        $request = "$method $request_url HTTP/" . $this->http . HTTP_NL;
        $request .= $this->_buildHeaders($headers);
        $request .= $this->_getCookies();
        $request .= HTTP_NL;
        $request .= $data;
        $this->_debug('request', $request);
        fputs($socket, $request);
        $r_headers = '';
        do {
            if (time() - $start > $this->timeout) {
                $this->status = -100;
                $this->error = sprintf('Timeout while reading headers (%.3fs)', $this->_time() - $this->start);
                return false;
            }
            if (feof($socket)) {
                $this->error = 'Premature End of File (socket)';
                return false;
            }
            $r_headers .= fgets($socket, 1024);
        } while (!preg_match('/\r?\n\r?\n$/', $r_headers));
        $this->_debug('response headers', $r_headers);
        if ($this->max_bodysize && preg_match('/\r?\nContent-Length:\s*(\d+)\r?\n/i', $r_headers, $match)) {
            if ($match[1] > $this->max_bodysize) {
                $this->error = 'Reported content length exceeds allowed response size';
                return false;
            }
        }
        if (!preg_match('/^HTTP\/(\d\.\d)\s*(\d+).*?\n/', $r_headers, $m)) {
            $this->error = 'Server returned bad answer';
            return false;
        }
        $this->status = $m[2];
        $this->resp_headers = $this->_parseHeaders($r_headers);
        if (isset($this->resp_headers['set-cookie'])) {
            foreach ((array)$this->resp_headers['set-cookie'] as $c) {
                list($key, $value, $foo) = explode('=', $cookie);
                $this->cookies[$key] = $value;
            }
        }
        $this->_debug('Object headers', $this->resp_headers);
        if ($this->status == 301 || $this->status == 302) {
            if (empty($this->resp_headers['location'])) {
                $this->error = 'Redirect but no Location Header found';
                return false;
            } elseif ($this->redirect_count == $this->max_redirect) {
                $this->error = 'Maximum number of redirects exceeded';
                return false;
            } else {
                $this->redirect_count++;
                $this->referer = $url;
                if (!preg_match('/^http/i', $this->resp_headers['location'])) {
                    $this->resp_headers['location'] = $uri['scheme'] . '://' . $uri['host'] . $this->resp_headers['location'];
                }
                return $this->sendRequest($this->resp_headers['location'], array(), 'GET');
            }
        }
        if ($this->header_regexp && !preg_match($this->header_regexp, $r_headers)) {
            $this->error = 'The received headers did not match the given regexp';
            return false;
        }
        $r_body = '';
        if (preg_match('/transfer\-(en)?coding:\s*chunked\r\n/i', $r_header)) {
            do {
                unset($chunk_size);
                do {
                    if (feof($socket)) {
                        $this->error = 'Premature End of File (socket)';
                        return false;
                    }
                    if (time() - $start > $this->timeout) {
                        $this->status = -100;
                        $this->error = sprintf('Timeout while reading chunk (%.3fs)', $this->_time() - $this->start);
                        return false;
                    }
                    $byte = fread($socket, 1);
                    $chunk_size .= $byte;
                } while (preg_match('/[a-zA-Z0-9]/', $byte));
                $byte = fread($socket, 1);
                $chunk_size = hexdec($chunk_size);
                $this_chunk = fread($socket, $chunk_size);
                $r_body .= $this_chunk;
                if ($chunk_size) $byte = fread($socket, 2);
                if ($this->max_bodysize && strlen($r_body) > $this->max_bodysize) {
                    $this->error = 'Allowed response size exceeded';
                    return false;
                }
            } while ($chunk_size);
        } else {
            while (!feof($socket)) {
                if (time() - $start > $this->timeout) {
                    $this->status = -100;
                    $this->error = sprintf('Timeout while reading response (%.3fs)', $this->_time() - $this->start);
                    return false;
                }
                $r_body .= fread($socket, 4096);
                $r_size = strlen($r_body);
                if ($this->max_bodysize && $r_size > $this->max_bodysize) {
                    $this->error = 'Allowed response size exceeded';
                    return false;
                }
                if ($this->resp_headers['content-length'] && !$this->resp_headers['transfer-encoding'] && $this->resp_headers['content-length'] == $r_size) {
                    break;
                }
            }
        }
        $status = socket_get_status($socket);
        fclose($socket);
        if ($this->resp_headers['content-encoding'] == 'gzip') {
            $this->resp_body = gzinflate(substr($r_body, 10));
        } else {
            $this->resp_body = $r_body;
        }
        $this->_debug('response body', $this->resp_body);
        $this->redirect_count = 0;
        return true;
    }

    function _debug($info, $var = null)
    {
        if (!$this->debug) return;
        if (!is_null($var)) {
            ob_start();
            $content = htmlspecialchars(ob_get_contents());
            ob_end_clean();
        }
    }

    function _time()
    {
        list($usec, $sec) = explode(" ", microtime());
        return ((float)$usec + (float)$sec);
    }

    function _parseHeaders($string)
    {
        $headers = array();
        $lines = explode("\n", $string);
        foreach ($lines as $line) {
            list($key, $val) = explode(':', $line, 2);
            $key = strtolower(trim($key));
            $val = trim($val);
            if (empty($val)) continue;
            if (isset($headers[$key])) {
                if (is_array($headers[$key])) {
                    $headers[$key][] = $val;
                } else {
                    $headers[$key] = array($headers[$key], $val);
                }
            } else {
                $headers[$key] = $val;
            }
        }
        return $headers;
    }

    function _buildHeaders($headers)
    {
        $string = '';
        foreach ($headers as $key => $value) {
            if (empty($value)) continue;
            $string .= $key . ': ' . $value . HTTP_NL;
        }
        return $string;
    }

    function _getCookies()
    {
        foreach ($this->cookies as $key => $val) {
            if ($headers) $headers .= '; ';
            $headers .= $key . '=' . $val;
        }
        if ($headers) $headers = "Cookie: $headers" . HTTP_NL;
        return $headers;
    }

    function _postEncode($data)
    {
        foreach ($data as $key => $val) {
            if ($url) $url .= '&';
            $url .= $key . '=' . urlencode($val);
        }
        return $url;
    }
}